/*
   _===_ _           _                  ______ _     _             _ _ _
  |  ___| |         | |                 | ___ (_)   | |           (_) | |
  | |__ | | ___  ___| |_ _ __ ___ ______| |_/ /_  __| | ___  _   _ _| | | ___ _   _ _ __
  |  __|| |/ _ \/ __| __| '__/ _ \______| ___ \ |/ _` |/ _ \| | | | | | |/ _ \ | | | '__|
  | |___| |  __/ (__| |_| | | (_) |     | |_/ / | (_| | (_) | |_| | | | |  __/ |_| | |
  \____/|_|\___|\___|\__|_|  \___/      \____/|_|\__,_|\___/ \__,_|_|_|_|\___|\__,_|_|


  Bidouilleur Arduino - Contrôle de relais via l'interface Ethernet ENC28J60
  Électro-Bidouilleur, Novembre 2020
*/ 

#include <EtherCard.h>   // la librairie Ethernet/ENC28J60 

#define STATIC 1  // mettre à 1 pour désactiver le DHCP (les variables myip/gwip doivent alors être assignées correctement)
#define Relai1 2
#define Relai2 3

#if STATIC
// ethernet interface ip address
static byte myip[] = { 192,168,0,67 };
// gateway ip address
static byte gwip[] = { 192,168,0,1 };
#endif

  // ethernet mac address - must be unique on your network
static byte mymac[] = { 0x74,0x69,0x69,0x2D,0x30,0x31 }; // l'adresse MAC ethernet
byte Ethernet::buffer[800]; // la mémoire tampon "buffer" d'envoi et de réception tcp/ip

  // Définition du contenu HTML à envoyer au client
  //Attention d'allouer suffisamment de mémoire dans le buffer tcp/ip pour contenir tout ce code HTML!
const char page[] PROGMEM =
"<html>"
  "<head>"
    "<meta http-equiv='Content-Type' content='text/html; charset=UTF-8'>"
    "<title>Contrôle de Relais É-B</title>"
  "</head>"
  "<body>"
    "<h3>Électro-Bidouilleur - Contrôle de Relais Arduino</h3>"
    "<a href='http://192.168.0.67?REL1=ON'><button>Relai 1 ON</button>"
    "<a href='http://192.168.0.67?REL1=OFF'><button>Relai 1 OFF</button><br>"
    "<a href='http://192.168.0.67?REL2=ON'><button>Relai 2 ON</button>"
    "<a href='http://192.168.0.67?REL2=OFF'><button>Relai 2 OFF</button><br>"
  "</body>"
"</html>"
;

  // Fonction d'initialisation exécutée une seule fois
void setup()
{
  pinMode(Relai1, OUTPUT);    // configurer la broche du relai 1 en sortie
  digitalWrite(Relai1, HIGH); // relâcher le relai 1
  pinMode(Relai2, OUTPUT);    // configurer la broche du relai 2 en sortie
  digitalWrite(Relai2, HIGH); // relâcher le relai 2
  
  Serial.begin(57600);  // Ouverture du port série pour le debug
  Serial.println("\n[Controle Relai É-B]");

  // 'SS' est prédéfini comme la broche Slave Select par défaut (broche 10). La changer si une autre broche est utilisée
  if (ether.begin(sizeof Ethernet::buffer, mymac, SS) == 0) // Ouvrir la communication avec le ENC28J60
    Serial.println( "Failed to access Ethernet controller");  // Problème de comm avec le ENC28J60

#if STATIC // Compilation avec adresse ip statique
  ether.staticSetup(myip, gwip);
#else // Compilation avec adresse ip dynamique (DHCP)
  if (!ether.dhcpSetup()) // Configurer le réseau via DHCP
    Serial.println("DHCP failed"); // Problème d'assignation d'adresse IP (réseau présent?)
#endif
    // Impression des paramètres de réseau sur le port série
  ether.printIp("IP:  ", ether.myip);
  ether.printIp("GW:  ", ether.gwip);
  ether.printIp("DNS: ", ether.dnsip);
}
  // La boucle principale
void loop()
{
  if (word pos = ether.packetLoop(ether.packetReceive())) // paquet TCP reçu?
  { // Oui, alors la mémoire tampon contient ici du data puisque pos n'est pas égal à zéro. 
    Serial.println((char *)(Ethernet::buffer + pos)); // Envoyer le contenu du tampon sur le port série.
    // Détection de quel relai est concerné par la commande
    if(strstr((char *)Ethernet::buffer + pos, "GET /?REL1=OFF") != 0)
    {
      Serial.println("Relai 1 Relâché");
      digitalWrite(Relai1, HIGH); 
    }
    else if(strstr((char *)Ethernet::buffer + pos, "GET /?REL1=ON") != 0)
    {
      Serial.println("Relai 1 Engagé");
      digitalWrite(Relai1, LOW);
    }
    else if(strstr((char *)Ethernet::buffer + pos, "GET /?REL2=OFF") != 0)
    {
      Serial.println("Relai 2 Relâché");
      digitalWrite(Relai2, HIGH);
    }
    else if(strstr((char *)Ethernet::buffer + pos, "GET /?REL2=ON") != 0)
    {
      Serial.println("Relai 2 Engagé");
      digitalWrite(Relai2, LOW);
    }
    // Envoi du contenu HTML vers le client
    memcpy_P(ether.tcpOffset(), page, sizeof page);
    ether.httpServerReply(sizeof page - 1);
  }
}
